function TCM_tangent
% TCM_TANGENT Interactive illustration of Tangent, Secant and Derivative.
%
%   TCM_TANGENT illustrates that the slope of the tangent can be used to define
%   the derivative of a function. If the tangent is approximated by a secant,
%   the resulting error is illustrated.
%
%   EXAMPLES
%   
%   Parabola: basic example for which the derivative is f(x)=2*x.
%    
%   Rational function: example with inflection points. 
%    
%   Trigonometric function: periodic example with many local extrema.
%
%   To investigate other cases, insert a function and its derivative
%   in the corresponding fields. If the <a href="http://www.mathworks.com/matlabcentral/fileexchange/61849-autodiff-r2016b">AutoDiff</a> toolbox is installed, 
%   the derivative can be computed automatically by leaving the field  
%   blank.
%
%   TCM_TANGENT is part of the <a href="http://www2.imng.uni-stuttgart.de/LstNumGeoMod/TCM/">TCM</a> project.
%   Related course materials can be found at <a href="http://www.mathematics-online.org/inhalt/aussage/aussage109">Mathematics Online</a>.

% Authors: Florian Martin, Lars von Wolff
% Last updated: 27.06.2016

% Initialize data
TCM_Data.points = [];
TCM_Data.funstring = '@(x) x.^2';
TCM_Data.derstring = '@(x) 2*x';
TCM_Data.maxerr = NaN;
TCM_Data.mode = 1;
TCM_Data.move = 0;

% Do not delete these Variables
TCM_Data.plot_window = [-2 2 -1.5 1.5];
TCM_Data.LmbMenuString = '';
TCM_Data.examples = {'Parabola','Rational function','Trigonometric function'};

% Layout:
% ---------------------------------------
% |               Title                 |
% | ------------ --------- ------------ |
% | | Settings | |Buttons| | Feedback | |
% | ------------ --------- ------------ |
% | ----------------------------------- |
% | |           Axis (Plot)           | |
% | ----------------------------------- |
% | Info-Button         Hint            |
% ---------------------------------------
% Initialize Layout:
TCM_UI.TitleHeightMin = 10;        % minimal height of the title
TCM_UI.TitleHeightMax = 32;        % maximal height of the title
TCM_UI.HintHeightMin = 8;          % minimal height of the hint
TCM_UI.HintHeightMax = 15;         % maximal height of the hint
TCM_UI.SettingsHeightMin = 40;     % minimal height of the feedback, buttons and 
                                   % setting panel
TCM_UI.SettingsHeightMax = 70;     % maximal height of the feedback, buttons and 
                                   % setting panel
TCM_UI.SettingsRatio = 4;          % aspect ratio of the settings  
                                   % (increase if more settings needed)
TCM_UI.ButtonsRatio = 1.4;         % aspect ratio of the Buttons panel
TCM_UI.FeedbackRatio = 2.6;        % aspect ratio of the feedback
TCM_UI.AxisHeightTarget = 0.70;    % desired fraction of figure 
                                   % used for the plot
TCM_UI.InfoPortion = 1/8;          % Portion of the Info Button in the last  
                                   % line 
TCM_UI.Title = 'Tangent and Derivative';
TCM_UI.mono = get(0,'FixedWidthFontName');
% Strings for generating the information window
TCM_UI.Author = 'F. Martin and L. von Wolff';
TCM_UI.Date = '27.06.2016';
TCM_UI.Web = 'http://www.mathematics-online.org/inhalt/aussage/aussage109';
      
CreateUI();

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% UI-Elements in Settings %%%%%%%%%%%%%%%%%%%%
% Popup for Function
TCM_UI.funtext = uicontrol('Units','normalized', 'Parent',TCM_UI.settingspanel, ...
        'BackgroundColor',get(gcf,'Color'), ...
        'FontUnits','normalized','FontSize',0.67,...
        'HorizontalAlignment','left', ...
	'Position',[.01,.70,.4,.30],...
	'String','Function:',...
	'Style','text');

% Input for Function
TCM_UI.funinput=uicontrol('Units','normalized', 'Parent',TCM_UI.settingspanel, ...
        'BackgroundColor',get(gcf,'Color'), ...
        'FontWeight','bold',...
        'FontUnits','normalized','FontSize',0.65,'FontName',TCM_UI.mono,...
        'HorizontalAlignment','left', ...
	'Position',[.45,.70,.5,.30],...
	'String','',...
	'Style','edit',...
	'Callback',@function_update);

% Text for Derivative Points
TCM_UI.dertext = uicontrol('Units','normalized', 'Parent',TCM_UI.settingspanel, ...
        'BackgroundColor',get(gcf,'Color'), ...
        'FontUnits','normalized','FontSize',0.67,...
        'HorizontalAlignment','left', ...
	'Position',[.01,.35,.4,.30],...
	'String','Derivative:',...
	'Style','text');

% Input for Derivative
TCM_UI.derinput=uicontrol('Units','normalized', 'Parent',TCM_UI.settingspanel, ...
        'BackgroundColor',get(gcf,'Color'), ...
        'FontWeight','bold',...
        'FontUnits','normalized','FontSize',0.65,'FontName',TCM_UI.mono,...
        'HorizontalAlignment','left', ...
	'Position',[.45,.35,.5,.30],...
	'String','',...
	'Style','edit',...
	'Callback',@function_update);

% Radiobutton Tangent
TCM_UI.trbutton=uicontrol('Units','normalized', 'Parent',TCM_UI.settingspanel, ...
        'BackgroundColor',get(gcf,'Color'), ...
        'FontWeight','bold',...
        'FontUnits','normalized','FontSize',0.67,...
        'HorizontalAlignment','left', ...
	'Position',[.1,.0,.4,.30],...
	'String','Tangent','Value',1,...
	'Style','radiobutton','Callback',{@changemode,1});

% Radiobutton Secant
TCM_UI.srbutton=uicontrol('Units','normalized', 'Parent',TCM_UI.settingspanel, ...
        'BackgroundColor',get(gcf,'Color'), ...
        'FontWeight','bold',...
        'FontUnits','normalized','FontSize',0.67,...
        'HorizontalAlignment','left', ...
	'Position',[.55,.0,.4,.30],...
	'String','Secant','Value',0,...
	'Style','radiobutton','Callback',{@changemode,2});

%%%%%%%%%%%%%%%%%%%%%%%%% UI-Elements in Buttons Panel %%%%%%%%%%%%%%%%%%%%
% Reset-Button
TCM_UI.reset=uicontrol('Units','normalized','Parent',TCM_UI.buttonspanel, ...
	'BackgroundColor',get(gcf,'Color'), ...
	'String','Reset', ...
    'FontUnits','normalized','FontSize',0.7,...
	'Position',[.01,.70,.98,.30], ...
	'Callback',@reset,...
	'Style','pushbutton');

% Example button
TCM_UI.examples=uicontrol('Units','normalized','Parent',TCM_UI.buttonspanel, ...
	'BackgroundColor',get(gcf,'Color'), ...
	'String','Examples', ...
    'FontUnits','normalized','FontSize',0.7,...
	'Position',[.01,.35,.98,.30], 'Callback',@OpenExampleMenu,...
	'Style','pushbutton');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% UI-Elements in Feedback %%%%%%%%%%%%%%%%%%%%

% Text for slope
TCM_UI.slopeval=uicontrol('Units','normalized','Parent',TCM_UI.feedbackpanel, ...
        'BackgroundColor',get(gcf,'Color'), ...
        'HorizontalAlignment','left', ...
        'FontUnits','normalized','FontSize',0.7,...
	'Position',[.1,.70,.9,.30],...
	'String','Slope: NaN',...
	'Style','text');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% UI-Elements in Axis %%%%%%%%%%%%%%%%%%%%%%%%
% Axis for Function display
TCM_UI.ax=axes('Position',[.05,.1,.42,.85],'Parent',TCM_UI.axispanel);
axis(TCM_UI.ax,'equal');
TCM_UI.modification = uicontextmenu('Callback',@reset_zoom_limits); 
TCM_UI.mpan = uimenu('Parent',TCM_UI.modification,'Label','Enable pan',...
        'Callback',@axis_context);
TCM_UI.mzoomi = uimenu('Parent',TCM_UI.modification,'Label','Enable zoom in',...
    'Callback',@axis_context); 
TCM_UI.mzoomo = uimenu('Parent',TCM_UI.modification,'Label','Enable zoom out',...
        'Callback',@axis_context); 
TCM_UI.ax.UIContextMenu = TCM_UI.modification;
TCM_UI.ax2=axes('Position',[.55,.1,.42,.85],'Parent',TCM_UI.axispanel);
TCM_UI.ax2.UIContextMenu = TCM_UI.modification;
linkaxes([TCM_UI.ax, TCM_UI.ax2], 'x');
% functions for smoothless zooming and panning
panfun = pan(TCM_UI.fig);
panfun.UIContextMenu = TCM_UI.modification;
set(panfun,'ActionPostCallback',@pan_modify);
zoomfun = zoom(TCM_UI.fig);
zoomfun.UIContextMenu = TCM_UI.modification; 
set(zoomfun,'ActionPostCallback',@zoom_modify);
set(zoomfun,'ButtonDownFilter',@set_zoom_limits);
set(zoomfun,'RightClickAction','PostContextMenu');
set(findall(TCM_UI.fig,'tag','Exploration.Pan'),'enable','off');
set(findall(TCM_UI.fig,'tag','Exploration.ZoomOut'),'enable','off');
set(findall(TCM_UI.fig,'tag','Exploration.ZoomIn'),'enable','off');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Start the program %%%%%%%%%%%%%%%%%%%%%%%%%%
changeexample(1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%% Callback functions %%%%%%%%%%%%%%%%%%%%%%%%%%%
function changemode(varargin)
if(TCM_Data.mode ~= varargin{end})
    TCM_Data.mode = varargin{end};
    reset;
else
    UpdateModeButtons;
end
end

% new function input
function function_update(varargin)
funchange = strcmp(TCM_Data.funstring,get(TCM_UI.funinput,'String'));
derchange = strcmp(TCM_Data.derstring,get(TCM_UI.derinput,'String'));
TCM_Data.funstring = get(TCM_UI.funinput,'String');
if (isempty(TCM_UI.derinput.String) || strcmp(TCM_UI.derinput.String,'automatic')...
        || (~funchange && derchange)) && exist('ainit','file')
    var = TCM_Data.funstring(find(TCM_Data.funstring=='(',1)+1:...
        find(TCM_Data.funstring==')',1)-1);
    TCM_Data.fun = eval([TCM_Data.funstring '+0*' var]);
    TCM_Data.der = adiff(TCM_Data.fun,1);
    set(TCM_UI.derinput,'String','automatic');
else
    TCM_Data.fun = eval(TCM_Data.funstring);
    TCM_Data.derstring = get(TCM_UI.derinput,'String');
    TCM_Data.der = eval(TCM_Data.derstring);
end
x = linspace(-2,+2);
f = TCM_Data.fun(x); f(isinf(f)) = 0;
hf = max(f)-min(f);
TCM_Data.plot_window = [x(1) x(end)];
TCM_Data.plot_window(3) = min(min(f)-hf/8,-1);
TCM_Data.plot_window(4) = max(max(f)+hf/8,1);
reset;
end

% refresh all data from UI
function reset(varargin)
% Disable Zoom/Pan
window_disable.Label = 'Disable pan';
axis_context(window_disable);
% Update UI
UpdateHinttext;
UpdateModeButtons;
% Reset the points
TCM_Data.points = [];
TCM_Data.move = 0;
% Update Plot
myplot;
end

% mouse button pressed 
function myDownfct(varargin)
if strcmp(get(TCM_UI.fig,'SelectionType'),'normal')
  p = getClickPosition(TCM_UI.ax);
  if(isnan(p))
      p = getClickPosition(TCM_UI.ax2);
      if(isnan(p))
          return;
      end
  end
  if((TCM_Data.mode == 1)||numel(TCM_Data.points) == 0)
      TCM_Data.points = p;
      TCM_Data.move = 1;
  elseif((TCM_Data.mode == 2)&&numel(TCM_Data.points) == 1)
      if(TCM_Data.points ~= p)
         TCM_Data.points = [TCM_Data.points p];
         TCM_Data.move = 2;
      end
  else
      [~,i]=min(abs(TCM_Data.points-p));
      TCM_Data.points(i)=p;
      TCM_Data.move = i;
  end
  myplot;
end
end

% moving the mouse in the axis
function myMovefct(varargin)
if TCM_Data.move > 0
    p = getClickPosition(TCM_UI.ax);
    if(isnan(p))
       p = getClickPosition(TCM_UI.ax2);
       if(isnan(p))
          return;
       end
    end
    if(numel(TCM_Data.points) == 1)
        TCM_Data.points(TCM_Data.move) = p;
    elseif(p ~=  TCM_Data.points(3-TCM_Data.move))
        TCM_Data.points(TCM_Data.move) = p;
    end
    myplot;
end
end

% releasing a mouse button
function myUpfct(varargin)
TCM_Data.move = 0;
myplot;
end

function p = getClickPosition(myaxes)
x = get(myaxes,'xlim');
y = get(myaxes,'ylim');
newpoint=get(myaxes,'CurrentPoint');
if (newpoint(1) < x(1) || newpoint(1) > x(2) || ...
    newpoint(3) < y(1) || newpoint(3) > y(2))
    p = NaN;
else
    p = newpoint(1);
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% kernel functions %%%%%%%%%%%%%%%%%%%%%%%%%%%

% plot function
function myplot
cla(TCM_UI.ax, 'reset'); 
hold(TCM_UI.ax, 'on');
cla(TCM_UI.ax2, 'reset'); 
hold(TCM_UI.ax2, 'on');
% evaluating and plotting of the input function on a square containing the
% current window for smoothless panning and zooming
h = TCM_Data.plot_window(2) - TCM_Data.plot_window(1);
x = linspace(TCM_Data.plot_window(1)-h,TCM_Data.plot_window(2)+h,2000);
fx = TCM_Data.fun(x);
derx = TCM_Data.der(x);
% plot of area for secant
if(TCM_Data.mode == 2 && numel(TCM_Data.points) == 2)
    p = sort(TCM_Data.points);
    slope = (TCM_Data.fun(p(2))-TCM_Data.fun(p(1)))/(p(2)-p(1));
    ind = x>p(1) & x<p(2);
    area(TCM_UI.ax2,[p(1) x(ind) p(2)],[ TCM_Data.der(p(1)) derx(ind) TCM_Data.der(p(2))],...
        slope,'EdgeColor','none','ShowBaseLine','off','FaceColor',[0.5 0.7 1]);
end
% plot of functions
plot(TCM_UI.ax,x,fx,'-','linewidth',2,'color',[0,0,.6]); 
plot(TCM_UI.ax,[x(1) x(end)],[0 0],'black');
plot(TCM_UI.ax2,x,derx,'-','linewidth',2,'color',[0,0,.6]); 
plot(TCM_UI.ax2,[x(1) x(end)],[0 0],'black');
% plot of the points
for i=1:numel(TCM_Data.points)
    plot(TCM_UI.ax,TCM_Data.points(i),TCM_Data.fun(TCM_Data.points(i)),'ored');
end
% plot of tangent
if(TCM_Data.mode == 1 && numel(TCM_Data.points) == 1)
    x0 = TCM_Data.points(1);
    xline = [x(1) x(end)];
    yline = TCM_Data.der(x0)*([x(1) x(end)]-x0)+TCM_Data.fun(x0);
    plot(TCM_UI.ax,xline,yline,'-','linewidth',1.25);
    plot(TCM_UI.ax2,x0,TCM_Data.der(x0),'ored');
    plot(TCM_UI.ax2,[x0 x0],[0 TCM_Data.der(x0)],'-','linewidth',1.25);
    %plot of slope triangle
    t1 = TCM_Data.fun(x0);
    t2 = TCM_Data.der(x0)+TCM_Data.fun(x0);
    tder = TCM_Data.der(x0);
    if( (TCM_Data.der(x0+1)- tder)*tder>0)
        plot(TCM_UI.ax,[x0 x0+1 x0+1],[t1,t1,t2],'linewidth',1.25);
        text(x0+1,(t1+t2)/2,num2str(tder,'%.2f'),'Parent',TCM_UI.ax,'Clipping','on',...
            'HorizontalAlignment','left','VerticalAlignment','middle');
        typos = t1;
    else
        plot(TCM_UI.ax,[x0 x0 x0+1],[t1,t2,t2],'linewidth',1.25);
        text(x0,(t1+t2)/2,num2str(tder,'%.2f'),'Parent',TCM_UI.ax,'Clipping','on',...
            'HorizontalAlignment','right','VerticalAlignment','middle');
        typos = t2;
    end
    if((TCM_Data.der(x0+1)- tder)>0)
        text(x0+1/2,typos,'1','Parent',TCM_UI.ax,'Clipping','on',...
            'HorizontalAlignment','center','VerticalAlignment','top');
    else
        text(x0+1/2,typos,'1','Parent',TCM_UI.ax,'Clipping','on',...
            'HorizontalAlignment','center','VerticalAlignment','bottom');
    end
    text(x0,tder/2,num2str(tder,'%.2f'),'Parent',TCM_UI.ax2,'Clipping','on',...
            'HorizontalAlignment','left','VerticalAlignment','middle');
    set(TCM_UI.slopeval,'String',sprintf('Slope: %.4f',tder));
% plot of single point (not secant)
elseif(TCM_Data.mode == 2 && numel(TCM_Data.points) == 1)
    x0 = TCM_Data.points(1);
    plot(TCM_UI.ax2,x0,TCM_Data.der(x0),'ored');
    set(TCM_UI.slopeval,'String','Slope: NaN');
% plot of secant
elseif(TCM_Data.mode == 2 && numel(TCM_Data.points) == 2)
    p = TCM_Data.points;
    xline = [x(1) x(end)];
    slope = (TCM_Data.fun(p(2))-TCM_Data.fun(p(1)))/(p(2)-p(1));
    yline = (xline-p(1))*slope + TCM_Data.fun(p(1));
    plot(TCM_UI.ax,xline,yline,'-','linewidth',1.25);
    plot(TCM_UI.ax2,[p(1) p(1) p(2) p(2)],[0 slope slope 0],'-','linewidth',1.25);
    set(TCM_UI.slopeval,'String',sprintf('Slope: %.4f',slope));
else
    set(TCM_UI.slopeval,'String','Slope: NaN');
end
SetWindowLimits
end

% Update the Hinttext
function UpdateHinttext
if(TCM_Data.mode == 1)
    LmbString = 'Left Mouse = Choose tangent point. ';
else
    if(numel(TCM_Data.points) > 1)
        LmbString = 'Left Mouse = Move secant point. ';
    else
        LmbString = 'Left Mouse = Choose secant point. ';
    end
end
if(numel(TCM_Data.LmbMenuString) == 0)
    set(TCM_UI.hinttext,'String',[LmbString 'Right Mouse = Access window menu.']);
else
    set(TCM_UI.hinttext,'String',[TCM_Data.LmbMenuString 'Right Mouse = Access window menu.']);
end
end

function UpdateModeButtons
if(TCM_Data.mode == 1)
    TCM_UI.trbutton.Value = 1;
    TCM_UI.srbutton.Value = 0;
else
    TCM_UI.trbutton.Value = 0;
    TCM_UI.srbutton.Value = 1;
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%% example functions %%%%%%%%%%%%%%%%%%%%%%%%%%%

function CreateExampleMenu
TCM_UI.UICexamples = uicontextmenu;
for i=1:numel(TCM_Data.examples)
    uimenu('Parent',TCM_UI.UICexamples,'Label',TCM_Data.examples{i},'Callback',{@changeexample,i});
end
end

function OpenExampleMenu(varargin)
TCM_UI.examples.Units = 'pixels';
TCM_UI.buttonspanel.Units = 'pixels';
TCM_UI.UICexamples.Position = TCM_UI.examples.Position(1:2) + TCM_UI.buttonspanel.Position(1:2);
TCM_UI.examples.Units = 'normalized';
TCM_UI.buttonspanel.Units = 'points';
TCM_UI.UICexamples.Visible = 'on';
end
 
              
function changeexample(varargin)  
switch varargin{numel(varargin)}
    case 1
        TCM_UI.funinput.String = '@(x) x.^2';
        TCM_UI.derinput.String = '@(x) 2*x';
        TCM_Data.plot_window = [-2 2 -2 4];
    case 2
        TCM_UI.funinput.String = '@(x) 1./(1+x.^2)';
        TCM_UI.derinput.String = '@(x) -2*x./(1+x.^2).^2';
        TCM_Data.plot_window = [-3 3 -0.5 1.5];
    case 3
        TCM_UI.funinput.String = '@(x) sin(x)+sin(3*x)/3';
        TCM_UI.derinput.String = '@(x) cos(x)+cos(3*x)';
        TCM_Data.plot_window = [0 2*pi -1.5 1.5];
end
TCM_Data.fun = eval(get(TCM_UI.funinput,'String'));
TCM_Data.der = eval(get(TCM_UI.derinput,'String'));
reset;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% zoom functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% function for updating the internal plot window
function zoom_modify(varargin)
if max(abs(TCM_Data.plot_window-axis(TCM_UI.ax)))< 10^-10
    ZoomOut;
end
pan_modify;
end

function pan_modify(varargin)
TCM_Data.plot_window = axis(TCM_UI.ax);
myplot;
end

% functions for proper zooming in (prevent double-click zoom out)
function [flag] = set_zoom_limits(varargin)
if(strcmp(TCM_UI.fig.SelectionType,'open'))
    flag = true;
else
    flag = false;
end
end

function reset_zoom_limits(varargin)
SetWindowLimits;
end

% handling the axis context menu
function axis_context(varargin)
zoom(TCM_UI.fig, 'off'); pan(TCM_UI.fig, 'off');
mode_strings = {'pan','zoom in','zoom out'};
status_strings = {'Enable ','Disable '};
switch varargin{1}.Label(1:7)
    case 'Enable '
        mode = strcmp(varargin{1}.Label(8:end),mode_strings);
        TCM_Data.LmbMenuString = ['Left Mouse = ' mode_strings{mode} ' the window. '];
        set(TCM_UI.mpan,'Label',horzcat(status_strings{mode(1)+1},mode_strings{1}));
        set(TCM_UI.mzoomi,'Label',horzcat(status_strings{mode(2)+1},mode_strings{2}));
        set(TCM_UI.mzoomo,'Label',horzcat(status_strings{mode(3)+1},mode_strings{3}));
        split_mode = strsplit(mode_strings{mode},' ');
        if numel(split_mode) == 1
            pan(TCM_UI.fig, 'on');
        else
            zoomh = zoom(TCM_UI.fig);
            zoomh.Enable = 'on';
            zoomh.Direction = split_mode{2};
        end
    case 'Disable'
        TCM_Data.LmbMenuString = '';
        set(TCM_UI.mpan,'Label','Enable pan');
        set(TCM_UI.mzoomi,'Label','Enable zoom in');
        set(TCM_UI.mzoomo,'Label','Enable zoom out');     
end
UpdateHinttext;
end

function SetWindowLimits
ZoomOut;
xlim(TCM_UI.ax,[TCM_Data.plot_window(1) TCM_Data.plot_window(2)]);
ylim(TCM_UI.ax,[TCM_Data.plot_window(3) TCM_Data.plot_window(4)]);
end

function ZoomOut
hx = TCM_Data.plot_window(2) - TCM_Data.plot_window(1);
hy = TCM_Data.plot_window(4) - TCM_Data.plot_window(3);
xlim(TCM_UI.ax,[TCM_Data.plot_window(1)-hx/2 TCM_Data.plot_window(2)+hx/2]);
ylim(TCM_UI.ax,[TCM_Data.plot_window(3)-hy/2 TCM_Data.plot_window(4)+hy/2]);
zoom(TCM_UI.fig, 'reset');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% layout functions %%%%%%%%%%%%%%%%%%%%%%%%%%%

% functions for creating the different UI-Parts and managing the resize of
% the window
function CreateUI
TCM_UI.fig = figure('Visible','off', 'Units','points','Toolbar','none',...
    'SizeChangedFcn',@calculate_layout, 'WindowButtonDownFcn',@myDownfct, ...
    'WindowButtonMotionFcn', @myMovefct,'WindowButtonUpFcn', @myUpfct,...
    'numbertitle','off','name',TCM_UI.Title);
TCM_UI.titletext = uicontrol('Units','points','BackgroundColor',TCM_UI.fig.Color, ...
	'HorizontalAlignment','center',  'FontWeight','bold', ...
    'FontUnits','normalized',        'FontSize',0.8,...
	'String',TCM_UI.Title,        'Style','text');
TCM_UI.hinttext = uicontrol('Units','points','BackgroundColor',TCM_UI.fig.Color, ...
    'HorizontalAlignment','right',...
    'FontUnits','normalized',        'FontSize',0.7,...
	'String','','Style','text');
TCM_UI.infobutton=uicontrol('Units','points', ...
	'BackgroundColor',TCM_UI.fig.Color, ...
	'String','Info', 'HorizontalAlignment','left',...
    'FontUnits','normalized','FontSize',0.7,...
	'Callback',@information,'Style','pushbutton');
TCM_UI.feedbackpanel = uipanel('Units','points','BackgroundColor',TCM_UI.fig.Color,'BorderType','none');
TCM_UI.buttonspanel = uipanel('Units','points','BackgroundColor',TCM_UI.fig.Color,'BorderType','none');
TCM_UI.settingspanel = uipanel('Units','points','BackgroundColor',TCM_UI.fig.Color,'BorderType','none');
TCM_UI.axispanel = uipanel('Units','points','BackgroundColor',TCM_UI.fig.Color,'BorderType','none');
CreateExampleMenu
TCM_UI.fig.Visible= 'on';
end


function calculate_layout(varargin) 
figpos = TCM_UI.fig.Position;
WMax = TCM_UI.SettingsHeightMax*(TCM_UI.SettingsRatio + TCM_UI.FeedbackRatio + TCM_UI.ButtonsRatio);
WMin = TCM_UI.SettingsHeightMin*(TCM_UI.SettingsRatio + TCM_UI.FeedbackRatio + TCM_UI.ButtonsRatio);
HMax = TCM_UI.TitleHeightMax + TCM_UI.HintHeightMax;
HMin = TCM_UI.TitleHeightMin + TCM_UI.HintHeightMin;

wratio = max(0,min(1,(min(figpos(3),1.3*figpos(4))-WMin)/(WMax-WMin)));
XCoords = [1,0,0,0,0,0,0,figpos(3)];
SWidth =      wratio*TCM_UI.SettingsHeightMax*TCM_UI.SettingsRatio...
        + (1-wratio)*TCM_UI.SettingsHeightMin*TCM_UI.SettingsRatio;
BWidth =      wratio*TCM_UI.SettingsHeightMax*TCM_UI.ButtonsRatio...
        + (1-wratio)*TCM_UI.SettingsHeightMin*TCM_UI.ButtonsRatio; 
FWidth =      wratio*TCM_UI.SettingsHeightMax*TCM_UI.FeedbackRatio...
        + (1-wratio)*TCM_UI.SettingsHeightMin*TCM_UI.FeedbackRatio; 
RemWidth = figpos(3)-3-SWidth-FWidth-BWidth;
XCoords(2) = max(1,0.3*RemWidth);
XCoords(3) = XCoords(2) + 1 + SWidth;
XCoords(4) = XCoords(3) + max(0,0.2*RemWidth);
XCoords(5) = XCoords(4) + 1 + BWidth;
XCoords(6) = XCoords(5) + max(0,0.2*RemWidth);
XCoords(7) = XCoords(6) + 1 + FWidth;
SHeight = wratio*TCM_UI.SettingsHeightMax + (1-wratio)*TCM_UI.SettingsHeightMin;
if(HMin > min(figpos(4),0.9*figpos(3))*(1-TCM_UI.AxisHeightTarget)-5-SHeight)
    hratio = 0;
elseif(HMax <  min(figpos(4),0.9*figpos(3))*(1-TCM_UI.AxisHeightTarget)-5-SHeight)
    hratio = 1;
else
    hratio = (( min(figpos(4),0.9*figpos(3))*(1-TCM_UI.AxisHeightTarget)-5-SHeight)  -HMin)/(HMax-HMin);
end
YCoords = [1,0,0,0,figpos(4)-1];
YCoords(2) = YCoords(1) + 1 + hratio*TCM_UI.HintHeightMax     + (1-hratio)*TCM_UI.HintHeightMin;
YCoords(4) = YCoords(5) - 1 - hratio*TCM_UI.TitleHeightMax    - (1-hratio)*TCM_UI.TitleHeightMin;
YCoords(3) = YCoords(4) - 1 - SHeight;
TCM_UI.titletext.Position =     [XCoords(1)+1,  YCoords(4)+1,  XCoords(8)-XCoords(1)-1,  YCoords(5)-YCoords(4)-1];
TCM_UI.settingspanel.Position = [XCoords(2)+1,  YCoords(3)+1,  XCoords(3)-XCoords(2)-1,  YCoords(4)-YCoords(3)-1];
TCM_UI.buttonspanel.Position =  [XCoords(4)+1,  YCoords(3)+1,  XCoords(5)-XCoords(4)-1,  YCoords(4)-YCoords(3)-1];
TCM_UI.feedbackpanel.Position = [XCoords(6)+1,  YCoords(3)+1,  XCoords(7)-XCoords(6)-1,  YCoords(4)-YCoords(3)-1];
TCM_UI.axispanel.Position =     [XCoords(1)+1,  YCoords(2)+1,  XCoords(8)-XCoords(1)-1,  max(0,YCoords(3)-YCoords(2)-1)];
TmpXCoord = TCM_UI.InfoPortion*(XCoords(8)-XCoords(1)-1);
TCM_UI.infobutton.Position =    [XCoords(1)+1,  YCoords(1)+1,  TmpXCoord-XCoords(1)-1,   YCoords(2)-YCoords(1)-1];
TCM_UI.hinttext.Position =      [TmpXCoord+1,   YCoords(1)+1,  XCoords(8)-TmpXCoord-1,   YCoords(2)-YCoords(1)-1];
end

% function for opening the information webpage
function information(varargin)
pos = TCM_UI.fig.OuterPosition;
infodialog = dialog('Name','Information','Units','points',...
    'OuterPosition',[pos(1)+pos(3)/4 pos(2)+pos(4)/4 pos(3)/2 pos(4)/2]);
uicontrol('Units','normalized', 'Parent',infodialog, ...
        'FontUnits','normalized', 'FontSize',0.67,...
        'HorizontalAlignment','center', 'Position',[.01,.6,.98,.15],...
	'String','Mathematics Online Link', 'Style','text',...
    'ForegroundColor','blue','ButtonDownFcn',@linkfun,...
    'Enable','inactive');
uicontrol('Units','normalized', 'Parent',infodialog, ...
        'FontUnits','normalized', 'FontSize',0.4,...
        'HorizontalAlignment','center', 'Position',[.01,.4,.98,.15],...
	'String',{horzcat('Created by ',TCM_UI.Author,'.');horzcat('Version: ',TCM_UI.Date)} , 'Style','text');
end

function linkfun(varargin)
web(TCM_UI.Web)
end
end